@@ -9,14 +9,16 @@
 # AUTHOR
 #   Andy Wardley   <abw@wardley.org>
 #
+#   Andrew Ford    <A.Ford@ford-mason.co.uk> (co-maintainer as of 03/2009)
+#
 # COPYRIGHT
-#   Copyright (C) 2000-2003 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 2000-2009 Andy Wardley.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
 # REVISION
-#   $Id: POM.pm,v 1.7 2003/07/24 15:40:59 abw Exp $
+#   $Id: POM.pm 66 2009-03-21 10:18:03Z ford $
 #
 #========================================================================
 
@@ -32,7 +34,7 @@ use Pod::POM::View::Pod;
 use vars qw( $VERSION $DEBUG $ERROR $ROOT $TEXTSEQ $DEFAULT_VIEW );
 use base qw( Exporter );
 
-$VERSION = 0.17;
+$VERSION = '0.24';
 $DEBUG   = 0 unless defined $DEBUG;
 $ROOT    = 'Pod::POM::Node::Pod';               # root node class
 $TEXTSEQ = 'Pod::POM::Node::Sequence';          # text sequence class
@@ -155,7 +157,7 @@ sub parse_file {
 
 sub parse_text {
     my ($self, $text, $name) = @_;
-    my ($para, $paralen, $gap, $type, $line, $inpod, $code, $result);
+    my ($para, $paralen, $gap, $type, $line, $inpod, $code, $result, $verbatim);
     my $warn = $self->{ WARNINGS } = [ ];
 
     my @stack = ( );
@@ -172,8 +174,9 @@ sub parse_text {
     $$line  = 1;
     $inpod  = 0;
 
-    
-    while ($text =~ /(?:(.*?)(\n{2,}))|(.+$)/sg) {
+# patch from JJ    
+#    while ($text =~ /(?:(.*?)(\n{2,}))|(.+$)/sg) {
+    while ($text =~ /(?:(.*?)((?:\s*\n){2,}))|(.+$)/sg) {
 	($para, $gap) = defined $1 ? ($1, $2) : ($3, '');
 
 	if ($para =~ s/^==?(\w+)\s*//) {
@@ -197,14 +200,53 @@ sub parse_text {
 	    $gap   = '';
 	}
 	elsif ($para =~ /^\s+/) {
-	    $type = 'verbatim';
+            $verbatim .= $para;
+            $verbatim .= $gap;
+            next;
 	}
 	else {
 	    $type = 'text';
 	    chomp($para);	    # catches last line in file
 	}
+
+        if ($verbatim) {
+    	    while(@stack) {
+                $verbatim =~ s/\s+$//s;
+       	        $result = $stack[-1]->add($self, 'verbatim', $verbatim);
+            
+	        if (! defined $result) {
+                    $self->warning($stack[-1]->error(), $name, $$line);
+                    undef $verbatim;
+		    last;
+	        }
+	        elsif (ref $result) {
+                    push(@stack, $result);
+                    undef $verbatim;
+		    last;
+	        }
+	        elsif ($result == REDUCE) {
+                    pop @stack;
+                    undef $verbatim;
+		    last;
+	        }
+	        elsif ($result == REJECT) {
+                    $self->warning($stack[-1]->error(), $name, $$line);
+		    pop @stack;
+	        }
+	        elsif (@stack == 1) {
+                    $self->warning("unexpected $type", $name, $$line);
+                    undef $verbatim;
+		    last;
+	        }
+	        else {
+                    pop @stack;
+	        }
+	    }
+        }
+
 	while(@stack) {
 	    $result = $stack[-1]->add($self, $type, $para);
+            
 	    if (! defined $result) {
 		$self->warning($stack[-1]->error(), $name, $$line);
 		last;
@@ -235,6 +277,41 @@ sub parse_text {
 	$$line += ($gap  =~ tr/\n//);
     }
 
+    if ($verbatim) {
+	while(@stack) {
+            $verbatim =~ s/\s+$//s;
+	    $result = $stack[-1]->add($self, 'verbatim', $verbatim);
+            
+	    if (! defined $result) {
+		$self->warning($stack[-1]->error(), $name, $$line);
+                undef $verbatim;
+		last;
+	    }
+	    elsif (ref $result) {
+		push(@stack, $result);
+                undef $verbatim;
+		last;
+	    }
+	    elsif ($result == REDUCE) {
+		pop @stack;
+                undef $verbatim;
+		last;
+	    }
+	    elsif ($result == REJECT) {
+		$self->warning($stack[-1]->error(), $name, $$line);
+		pop @stack;
+	    }
+	    elsif (@stack == 1) {
+		$self->warning("unexpected $type", $name, $$line);
+                undef $verbatim;
+		last;
+	    }
+	    else {
+		pop @stack;
+	    }
+	}
+    }
+
     return $stack[0];
 }
 
@@ -428,7 +505,7 @@ Pod::POM - POD Object Model
         || die $parser->error();
 
     # parse from a file specified by name or filehandle
-    my $pom = $parser->parse_text($file)
+    my $pom = $parser->parse_file($file)
         || die $parser->error();
 
     # parse from text or file 
@@ -1511,13 +1588,15 @@ carefully.
 
 Andy Wardley E<lt>abw@kfs.orgE<gt>
 
+Andrew Ford E<lt>A.Ford@ford-mason.co.ukE<gt> (co-maintainer as of 03/2009)
+
 =head1 VERSION
 
-This is version 0.15 of the Pod::POM module.
+This is version 0.24 of the Pod::POM module.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2000-2002 Andy Wardley.  All Rights Reserved.
+Copyright (C) 2000-2009 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
